// LabWindows/CVI Example using rsscope VXIpnp driver
// Notes:      To run the example you need to install rsscope VXI Plug&Play
//             instrument driver version 3.35.0 or later.
//             Default locations of instrument driver files on 32bit Windows:
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\rsscope\rsscope.fp
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\include\rsscope.h
//             C:\Program Files (x86)\IVI Foundation\VISA\WinNT\lib\msc\rsscope.lib
//             Default locations of instrument driver files on 64bit Windows:
//             C:\Program Files\IVI Foundation\VISA\Win64\rsscope\rsscope.fp
//             C:\Program Files\IVI Foundation\VISA\Win64\include\rsscope.h
//             C:\Program Files\IVI Foundation\VISA\Win64\lib\msc\rsscope.lib

#include "rsscope.h"
#include <utility.h>
#include <ansi_c.h>

// error checking macro
#define CHECKERR(fCal) \
	status = (fCal);if (status < VI_SUCCESS) goto ERROR_HANDLING;

#define ARRAY_SIZES 30000

    

int main()
{
	ViSession scope;
	ViStatus status = VI_SUCCESS;
    ViInt32 errorCode;
	ViChar response[1024];
    ViChar  error_message [1024];
    ViChar  error_buffer  [16384];
    ViChar* p2buf;
	ViReal64 waveform[ARRAY_SIZES];
	ViInt32 count;
    ViInt32 opc;
    ViReal64 initialX, incrementX;
       

	//-----------------------------------------------------------
	//Initialization:
	//-----------------------------------------------------------
    SetBreakOnLibraryErrors (0); // Don't show Pupup messages with errors from libraries.
    CHECKERR(rsscope_init("TCPIP::192.168.1.101::INSTR", VI_TRUE, VI_FALSE, &scope)); // Adjust the VISA Resource string to fit your instrument
	CHECKERR(rsscope_IDQueryResponse(scope, response));
	CHECKERR(rsscope_reset(scope)); // Reset the instrument, clear the Error queue
	CHECKERR(rsscope_ConfigureDisplayUpdate(scope,VI_TRUE)); // Display update ON - switch OFF after debugging
	//-----------------------------------------------------------
	// Basic Settings:
	//---------------------------- -------------------------------
	                  
    CHECKERR(rsscope_ConfigureTimeSelectDependency (scope, RSSCOPE_VAL_RECORD_TIME)); // Define Horizontal scale by time
	CHECKERR(rsscope_ConfigureAcquisitionTime(scope, 0.01)); // 10ms Acquisition time
	CHECKERR(rsscope_ConfigureRecordLength(scope, 20002)); // 20002 X points
	CHECKERR(rsscope_ConfigureChannel(scope, RSSCOPE_VAL_CHANNEL_1, VI_TRUE, 2.0, 0.0, RSSCOPE_VAL_AC)); // Horizontal range 2V, AC coupling
	//-----------------------------------------------------------
	// Trigger Settings:
	//-----------------------------------------------------------
	CHECKERR(rsscope_ConfigureTriggerModifier(scope, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_AUTO)); // Trigger Auto mode in case of no signal is applied
    CHECKERR(rsscope_ConfigureTrigger(scope, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_EDGE_TRIGGER));  // Trigger type Edge 
    CHECKERR(rsscope_ConfigureEdgeTriggerSource(scope, RSSCOPE_VAL_TRIGGER_A, RSSCOPE_VAL_CHANNEL1, 0.04, RSSCOPE_VAL_POSITIVE)); // Source CH1, Level 40mV, Positive Edge
    CHECKERR(rsscope_QueryOPC(scope, &opc)); //Using *OPC? query waits until all the instrument settings are finished
	// -----------------------------------------------------------
	// SyncPoint 'SettingsApplied' - all the settings were applied
	// -----------------------------------------------------------
	// Arming the SCOPE for single acquisition
	// -----------------------------------------------------------
    CHECKERR(rsscope_InitiateAcquisition(scope, RSSCOPE_VAL_INIT_SINGLE_WITHOUT_WAIT));
	// -----------------------------------------------------------
	// DUT_Generate_Signal() - in our case we use Probe compensation signal
	// where the trigger event (positive edge) is reoccuring
	// -----------------------------------------------------------
	CHECKERR(rsscope_WaitForMeasurementComplete(scope, 2000)); // Wait until the instrument finished the Acquisition, timeout 2000ms
	// -----------------------------------------------------------
	// SyncPoint 'AcquisitionFinished' - the results are ready
	// -----------------------------------------------------------
	// Fetching the waveform - only Binary format is available in drivers
	// The waveformBIN data and waveformASC data are however the same.
	// -----------------------------------------------------------
	printf("Fetching waveform in Binary format... ");
	CHECKERR(rsscope_FetchWaveform(scope, RSSCOPE_VAL_CHANNEL_1, 1, ARRAY_SIZES, waveform, &count, &initialX, &incrementX));
    printf("Samples count: %d\n", count);
	// -----------------------------------------------------------
	// Making an instrument screenshot and transferring the file to the PC
	// -----------------------------------------------------------
	CHECKERR(rsscope_ConfigureHardcopyDevice(scope, 1, RSSCOPE_VAL_HARDCOPY_DESTINATION_PNG, VI_TRUE, RSSCOPE_VAL_HARDCOPY_ORIENTATION_LANDSCAPE)); // Set the screenshot format
	CHECKERR(rsscope_HardcopySetFileName(scope, "c:\\temp\\Dev_Screenshot.png")); // Set the screenshot path
	CHECKERR(rsscope_HardcopyPrint(scope)); // Make the screenshot now
    CHECKERR(rsscope_ReadToFileFromInstrument(scope, "c:\\temp\\Dev_Screenshot.png", "c:\\temp\\PC_Screenshot.png")); // Copy the file from instrument to the PC
	printf("\nScreenshot file saved to PC 'c:\\Temp\\PC_Screenshot.png'");

ERROR_HANDLING:
	if (status == VI_SUCCESS)
	{
		; // all went OK
	}
	else if (status < VI_SUCCESS)
    {
        rsscope_error_message (scope, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        /* This function reads an error code and a message from the instrument's error queue */
        rsscope_error_query (scope, &errorCode, error_message);
        
        if (errorCode != 0)
        {
            sprintf (p2buf, "Secondary Error: Instrument is reporting the following error (SYST:ERR?): %d, %s\n", errorCode, error_message);
        }
        
        printf ("\nERROR!\n\n");
        printf (error_buffer);
    }
        
    rsscope_close (scope);
    
    
    printf ("\nPress ENTER to continue...");
    (void) getchar();
        
    return status;
}
